#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.antigua_and_barbuda import AntiguaAndBarbuda
from tests.common import CommonCountryTests


class TestAntiguaAndBarbuda(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(AntiguaAndBarbuda)

    def test_special_holidays(self):
        name_day_after_general_election = "Day after the General Election"
        for dt, name in (
            ("1993-08-03", "Public Holiday"),
            ("2008-02-19", "State Funeral of the late The Honourable Charlesworth T. Samuel"),
            ("2008-03-18", "State Funeral of the late The Honourable Sir George Herbert Walter"),
            ("2018-03-22", name_day_after_general_election),
            ("2023-01-19", name_day_after_general_election),
        ):
            self.assertHolidayName(name, dt)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "1995-01-02",
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(
            name,
            "2021-05-03",
            "2022-05-02",
            "2023-05-01",
            "2024-05-06",
            "2025-05-05",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_caribbean_community_day(self):
        name = "Caribbean Community (Caricom) Day"
        self.assertHolidayName(
            name,
            "1995-07-03",
            "2000-07-03",
            "2004-07-05",
            "2005-07-04",
        )
        self.assertHolidayName(name, range(self.start_year, 2006))
        self.assertNoHolidayName(name, range(2006, self.end_year))

    def test_carnival_monday(self):
        name = "Carnival Monday"
        self.assertHolidayName(
            name,
            "2021-08-02",
            "2022-08-01",
            "2023-08-07",
            "2024-08-05",
            "2025-08-04",
        )
        self.assertHolidayName(name, self.full_range)

    def test_carnival_tuesday(self):
        name = "Carnival Tuesday"
        self.assertHolidayName(
            name,
            "2021-08-03",
            "2022-08-02",
            "2023-08-08",
            "2024-08-06",
            "2025-08-05",
        )
        self.assertHolidayName(name, range(2006, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2006))

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-11-01" for year in self.full_range))
        obs_dts = (
            "2008-11-03",
            "2009-11-02",
            "2014-11-03",
            "2015-11-02",
            "2020-11-02",
            "2025-11-03",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_heroes_and_sir_vere_cornwall_bird_snr_day(self):
        name_2005 = "National Heroes Day"
        name_2014 = "Sir Vere Cornwall Bird Snr. Day"
        self.assertHolidayName(
            name_2005,
            "2005-12-09",
            "2006-12-11",
            "2007-12-09",
            "2008-12-09",
            "2009-12-09",
            "2010-12-09",
            "2011-12-09",
            "2012-12-10",
            "2013-12-09",
        )
        self.assertHolidayName(name_2014, (f"{year}-12-09" for year in range(2014, self.end_year)))
        self.assertNoHolidayName(
            name_2005, range(self.start_year, 2005), range(2014, self.end_year)
        )
        self.assertNoHolidayName(name_2014, range(self.start_year, 2014))

        obs_dts = ("2023-12-11",)
        self.assertHolidayName(f"{name_2014} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2004-12-27",
            "2005-12-27",
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2004-12-28",
            "2010-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-05", "Labour Day"),
            ("2025-06-09", "Whit Monday"),
            ("2025-08-04", "Carnival Monday"),
            ("2025-08-05", "Carnival Tuesday"),
            ("2025-11-01", "Independence Day"),
            ("2025-11-03", "Independence Day (observed)"),
            ("2025-12-09", "Sir Vere Cornwall Bird Snr. Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.south_georgia_and_the_south_sandwich_islands import (
    SouthGeorgiaAndTheSouthSandwichIslands,
)
from tests.common import CommonCountryTests


class TestSouthGeorgiaAndTheSouthSandwichIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(2012, 2050)
        super().setUpClass(
            SouthGeorgiaAndTheSouthSandwichIslands, years=years, years_non_observed=years
        )

    def test_special_holidays(self):
        self.assertHoliday(
            "2022-06-02",
            "2022-06-03",
            "2023-05-08",
            "2023-11-14",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(2012, 2050)))
        obs_dt = (
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_possession_day(self):
        name = "Possession Day"
        self.assertHolidayName(name, (f"{year}-01-17" for year in range(2012, 2050)))
        obs_dt = (
            "2015-01-19",
            "2016-01-18",
            "2021-01-18",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(2012, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2012-04-09",
            "2013-04-01",
            "2014-04-21",
            "2015-04-06",
            "2016-03-28",
            "2017-04-17",
            "2018-04-02",
            "2019-04-22",
        )
        self.assertNoHolidayName(name, range(2020, 2050))

    def test_queens_birthday(self):
        name = "The Queen's Birthday"
        self.assertHolidayName(name, (f"{year}-04-21" for year in range(2018, 2023)))
        self.assertNoHolidayName(name, range(2012, 2018), range(2023, 2050))
        obs_dt = (
            "2018-04-23",
            "2019-04-23",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_liberation_day(self):
        name = "Liberation Day"
        self.assertHolidayName(
            name,
            (f"{year}-04-26" for year in range(2012, 2016)),
            (f"{year}-04-25" for year in range(2016, 2050)),
        )
        obs_dt = (
            "2014-04-28",
            "2015-04-27",
            "2020-04-27",
            "2021-04-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_shackleton_day(self):
        name = "Shackleton Day"
        self.assertHolidayName(name, (f"{year}-05-20" for year in range(2015, 2050)))
        self.assertNoHolidayName(name, range(2012, 2015))
        obs_dt = (
            "2017-05-22",
            "2018-05-21",
            "2023-05-19",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_mid_winter_day(self):
        name = "Mid-winter Day"
        self.assertHolidayName(name, (f"{year}-06-21" for year in range(2012, 2050)))
        obs_dt = (
            "2014-06-23",
            "2015-06-22",
            "2020-06-22",
            "2025-06-20",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_toothfish_day(self):
        name_1 = "Toothfish (end of season) Day"
        name_2 = "Toothfish Day"
        self.assertHolidayName(name_1, (f"{year}-09-14" for year in range(2012, 2015)))
        self.assertHolidayName(name_2, (f"{year}-09-04" for year in range(2015, 2022)))
        self.assertNoHolidayName(name_1, range(2015, 2050))
        self.assertNoHolidayName(name_2, range(2012, 2015), range(2022, 2050))
        obs_dt_1 = (
            "2013-09-13",
            "2014-09-15",
        )
        obs_dt_2 = (
            "2016-09-05",
            "2021-09-06",
        )
        self.assertHolidayName(f"{name_1} (observed)", obs_dt_1)
        self.assertHolidayName(f"{name_2} (observed)", obs_dt_2)
        self.assertNoNonObservedHoliday(obs_dt_1, obs_dt_2)

    def test_environment_day(self):
        name = "Environment Day"
        self.assertHolidayName(name, (f"{year}-10-30" for year in range(2020, 2050)))
        self.assertNoHolidayName(name, range(2012, 2020))
        obs_dt = (
            "2021-10-29",
            "2022-10-31",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(2012, 2050)))
        obs_dt = (
            "2015-12-28",
            "2016-12-27",
            "2020-12-28",
            "2021-12-28",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(2012, 2050)))
        obs_dt = (
            "2016-12-26",
            "2021-12-27",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-01-17", "Possession Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-25", "Liberation Day"),
            ("2023-05-08", "Coronation of King Charles III"),
            ("2023-05-19", "Shackleton Day (observed)"),
            ("2023-05-20", "Shackleton Day"),
            ("2023-06-21", "Mid-winter Day"),
            ("2023-10-30", "Environment Day"),
            ("2023-11-14", "King's Birthday"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-01-17", "Possession Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-25", "Liberation Day"),
            ("2023-05-08", "King Charles III's Coronation"),
            ("2023-05-19", "Shackleton Day (observed)"),
            ("2023-05-20", "Shackleton Day"),
            ("2023-06-21", "Mid-winter Day"),
            ("2023-10-30", "Environment Day"),
            ("2023-11-14", "King's Birthday"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.british_indian_ocean_territory import BritishIndianOceanTerritory
from tests.common import CommonCountryTests


class TestBritishIndianOceanTerritory(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        cls.full_range = range(1900, 2050)
        super().setUpClass(BritishIndianOceanTerritory)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.iraq import Iraq
from tests.common import CommonCountryTests


class TestIraq(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Iraq)

    def test_new_years_day(self):
        name = "رأس السنة الميلادية"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1973, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1973))
        self.assertChristianHolidayName(
            name, (f"{year}-01-01" for year in range(self.start_year, 1973))
        )
        self.assertNoChristianHolidayName(name, range(1973, self.end_year))

    def test_army_day(self):
        self.assertHolidayName(
            "عيد الجيش", (f"{year}-01-06" for year in range(1973, self.end_year))
        )

    def test_february_8_revolution_day(self):
        name = "ثورة 8 شباط"
        self.assertHolidayName(name, (f"{year}-02-08" for year in range(self.start_year, 2025)))
        self.assertNoHolidayName(name, range(2025, self.end_year))

    def test_commemoration_of_saddam_baath_crimes_against_iraqi_people(self):
        name = "ذكرى جرائم البعث والأنفال والهجوم على حلبجة"
        self.assertHolidayName(name, (f"{year}-03-16" for year in range(2025, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2025))

    def test_nowruz(self):
        name = "عيد نوروز"
        self.assertHolidayName(name, (f"{year}-03-21" for year in range(1969, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1969))

    def test_labor_day(self):
        self.assertHolidayName(
            "عيد العمال العالمي", (f"{year}-05-01" for year in range(1973, self.end_year))
        )

    def test_july_14_revolution_day(self):
        name = "ثورة 14 تموز"
        self.assertHolidayName(name, (f"{year}-07-14" for year in range(self.start_year, 2024)))
        self.assertNoHolidayName(name, range(2024, self.end_year))

    def test_july_17_revolution_day(self):
        name = "ثورة 17 تموز"
        self.assertHolidayName(name, (f"{year}-07-17" for year in range(1969, 2024)))
        self.assertNoHolidayName(name, range(self.start_year, 1969), range(2024, self.end_year))

    def test_islamic_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_ashura(self):
        name = "عاشوراء"
        self.assertHolidayName(
            name,
            "2021-08-19",
            "2022-08-08",
            "2023-07-29",
            "2024-07-16",
            "2025-07-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophets_birthday(self):
        name = "المولد النبوي الشريف"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        self.assertHolidayName(
            name,
            "2023-04-21",
            "2023-04-22",
            "2023-04-23",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
            "2025-03-31",
            "2025-04-01",
            "2025-04-02",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_day_of_arafah(self):
        name = "يوم عرفة"
        self.assertHolidayName(
            name,
            "2020-07-30",
            "2021-07-19",
            "2022-07-08",
            "2023-06-27",
            "2024-06-15",
            "2025-06-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        self.assertHolidayName(
            name,
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
            "2025-06-06",
            "2025-06-07",
            "2025-06-08",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_ghadir(self):
        name = "عيد الغدير"
        self.assertHolidayName(
            name,
            "2024-06-24",
            "2025-06-15",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(2024, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 2024))

    def test_easter_sunday(self):
        name = "أحد الفصح"
        self.assertNoHolidayName(name)
        self.assertChristianHolidayName(
            name,
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertChristianHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "إثنين الفصح"
        self.assertNoHolidayName(name)
        self.assertChristianHolidayName(
            name,
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertChristianHolidayName(name, self.full_range)

    def test_christmas_day(self):
        name = "عيد الميلاد"
        self.assertNoHolidayName(name)
        self.assertChristianHolidayName(name, (f"{year}-12-25" for year in self.full_range))

    def test_pesach(self):
        name = "عيد الفصح"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "2021-03-28",
            "2021-03-29",
            "2022-04-16",
            "2022-04-17",
            "2023-04-06",
            "2023-04-07",
        )
        self.assertHebrewHolidayName(name, range(self.start_year, 2024))
        self.assertNoHebrewHolidayName(name, range(2024, self.end_year))

    def test_yom_kippur(self):
        name = "يوم الكفارة"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "2021-09-16",
            "2022-10-05",
            "2023-09-25",
        )
        self.assertHebrewHolidayName(name, range(self.start_year, 2024))
        self.assertNoHebrewHolidayName(name, range(2024, self.end_year))

    def test_sukkot(self):
        name = "عيد المظلة"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "2021-09-21",
            "2021-09-22",
            "2022-10-10",
            "2022-10-11",
            "2023-09-30",
            "2023-10-01",
        )
        self.assertHebrewHolidayName(name, range(self.start_year, 2024))
        self.assertNoHebrewHolidayName(name, range(2024, self.end_year))

    def test_great_feast(self):
        name = "يوما عيد البنجة"
        self.assertNoHolidayName(name)
        self.assertSabianHolidayName(
            name,
            "2023-07-22",
            "2023-07-23",
            "2024-07-21",
            "2024-07-22",
            "2024-07-23",
            "2024-07-24",
        )
        self.assertSabianHolidayNameCount(name, 2, range(1973, 2024))
        self.assertSabianHolidayNameCount(name, 4, range(2024, self.end_year))
        self.assertNoSabianHolidayName(name, range(self.start_year, 1973))

    def test_parwanaya(self):
        name = "عيد الخليقة"
        self.assertNoHolidayName(name)
        self.assertSabianHolidayName(
            name,
            "2023-03-14",
            "2023-03-15",
            "2024-03-13",
            "2024-03-14",
            "2024-03-15",
            "2024-03-16",
            "2024-03-17",
        )
        self.assertSabianHolidayNameCount(name, 2, range(1973, 2024))
        self.assertSabianHolidayNameCount(name, 5, range(2024, self.end_year))
        self.assertNoSabianHolidayName(name, range(self.start_year, 1973))
        self.assertNoSabianHolidayName(
            name,
            "2023-03-16",
            "2023-03-17",
            "2023-03-18",
        )

    def test_little_feast(self):
        name = "عيد الصغير"
        self.assertNoHolidayName(name)
        self.assertSabianHolidayName(
            name,
            "2023-04-10",
            "2024-04-09",
            "2024-04-10",
            "2025-04-09",
            "2025-04-10",
        )
        self.assertSabianHolidayNameCount(name, 1, range(1973, 2024))
        self.assertSabianHolidayNameCount(name, 2, range(2024, self.end_year))
        self.assertNoSabianHolidayName(name, range(self.start_year, 1973))

    def test_prophet_yahyas_birthday(self):
        name = "مولد النبي يحيى عليه السلام"
        self.assertNoHolidayName(name)
        self.assertSabianHolidayName(
            name,
            "2021-05-18",
            "2022-05-18",
            "2023-05-18",
            "2024-05-17",
            "2025-05-17",
        )
        self.assertSabianHolidayName(name, range(1973, self.end_year))
        self.assertNoSabianHolidayName(name, range(self.start_year, 1973))

    def test_yazidi_new_year(self):
        name = "رأس السنة الإيزيدية"
        self.assertNoHolidayName(name)
        self.assertYazidiHolidayName(
            name,
            "2021-04-14",
            "2022-04-20",
            "2023-04-19",
            "2024-04-17",
            "2025-04-16",
        )
        self.assertYazidiHolidayName(name, self.full_range)

    def test_yazidi_summer_festival(self):
        name = "مهرجان الصيف اليزيدي"
        self.assertNoHolidayName(name)
        self.assertYazidiHolidayName(
            name,
            (f"{year}-07-31" for year in range(self.start_year, 2024)),
            (f"{year}-08-01" for year in range(self.start_year, 2024)),
            (f"{year}-08-02" for year in self.full_range),
            (f"{year}-08-03" for year in self.full_range),
        )

    def test_feast_of_the_assembly(self):
        name = "عيد الجمعية"
        self.assertNoHolidayName(name)
        self.assertYazidiHolidayName(
            name,
            (f"{year}-10-06" for year in self.full_range),
            (f"{year}-10-07" for year in self.full_range),
            (f"{year}-10-08" for year in self.full_range),
            (f"{year}-10-09" for year in self.full_range),
            (f"{year}-10-10" for year in self.full_range),
            (f"{year}-10-11" for year in self.full_range),
            (f"{year}-10-12" for year in self.full_range),
            (f"{year}-10-13" for year in self.full_range),
        )

    def test_feast_of_ezi(self):
        name = "عيد إيزي"
        self.assertNoHolidayName(name)
        self.assertYazidiHolidayName(
            name,
            "2021-12-17",
            "2022-12-16",
            "2023-12-15",
            "2024-12-20",
            "2025-12-19",
        )
        self.assertYazidiHolidayName(name, self.full_range)

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "رأس السنة الميلادية"),
            ("2024-01-06", "عيد الجيش"),
            ("2024-02-08", "ثورة 8 شباط"),
            ("2024-03-21", "عيد نوروز"),
            ("2024-04-10", "عيد الفطر"),
            ("2024-04-11", "عيد الفطر"),
            ("2024-04-12", "عيد الفطر"),
            ("2024-05-01", "عيد العمال العالمي"),
            ("2024-06-15", "يوم عرفة"),
            ("2024-06-16", "عيد الأضحى"),
            ("2024-06-17", "عيد الأضحى"),
            ("2024-06-18", "عيد الأضحى"),
            ("2024-06-24", "عيد الغدير"),
            ("2024-07-07", "رأس السنة الهجرية"),
            ("2024-07-16", "عاشوراء"),
            ("2024-09-15", "المولد النبوي الشريف"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "رأس السنة الميلادية"),
            ("2025-01-06", "عيد الجيش"),
            ("2025-03-13", "عيد الخليقة"),
            ("2025-03-14", "عيد الخليقة"),
            ("2025-03-15", "عيد الخليقة"),
            ("2025-03-16", "ذكرى جرائم البعث والأنفال والهجوم على حلبجة; عيد الخليقة"),
            ("2025-03-17", "عيد الخليقة"),
            ("2025-03-21", "عيد نوروز"),
            ("2025-03-31", "عيد الفطر"),
            ("2025-04-01", "عيد الفطر"),
            ("2025-04-02", "عيد الفطر"),
            ("2025-04-09", "عيد الصغير"),
            ("2025-04-10", "عيد الصغير"),
            ("2025-04-16", "رأس السنة الإيزيدية"),
            ("2025-04-20", "أحد الفصح"),
            ("2025-04-21", "إثنين الفصح"),
            ("2025-05-01", "عيد العمال العالمي"),
            ("2025-05-17", "مولد النبي يحيى عليه السلام"),
            ("2025-06-05", "يوم عرفة"),
            ("2025-06-06", "عيد الأضحى"),
            ("2025-06-07", "عيد الأضحى"),
            ("2025-06-08", "عيد الأضحى"),
            ("2025-06-15", "عيد الغدير"),
            ("2025-06-26", "رأس السنة الهجرية"),
            ("2025-07-05", "عاشوراء"),
            ("2025-07-21", "يوما عيد البنجة"),
            ("2025-07-22", "يوما عيد البنجة"),
            ("2025-07-23", "يوما عيد البنجة"),
            ("2025-07-24", "يوما عيد البنجة"),
            ("2025-08-02", "مهرجان الصيف اليزيدي"),
            ("2025-08-03", "مهرجان الصيف اليزيدي"),
            ("2025-09-04", "المولد النبوي الشريف"),
            ("2025-10-06", "عيد الجمعية"),
            ("2025-10-07", "عيد الجمعية"),
            ("2025-10-08", "عيد الجمعية"),
            ("2025-10-09", "عيد الجمعية"),
            ("2025-10-10", "عيد الجمعية"),
            ("2025-10-11", "عيد الجمعية"),
            ("2025-10-12", "عيد الجمعية"),
            ("2025-10-13", "عيد الجمعية"),
            ("2025-12-19", "عيد إيزي"),
            ("2025-12-25", "عيد الميلاد"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-06", "Army Day"),
            ("2025-03-13", "Feast of Creation"),
            ("2025-03-14", "Feast of Creation"),
            ("2025-03-15", "Feast of Creation"),
            (
                "2025-03-16",
                "Commemoration of the Saddam Baath crimes against the Iraqi people"
                "; Feast of Creation",
            ),
            ("2025-03-17", "Feast of Creation"),
            ("2025-03-21", "Nowruz"),
            ("2025-03-31", "Eid al-Fitr"),
            ("2025-04-01", "Eid al-Fitr"),
            ("2025-04-02", "Eid al-Fitr"),
            ("2025-04-09", "Little Feast"),
            ("2025-04-10", "Little Feast"),
            ("2025-04-16", "Yazidi New Year"),
            ("2025-04-20", "Easter Sunday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-17", "Prophet Yahya's Birthday"),
            ("2025-06-05", "Day of Arafah"),
            ("2025-06-06", "Eid al-Adha"),
            ("2025-06-07", "Eid al-Adha"),
            ("2025-06-08", "Eid al-Adha"),
            ("2025-06-15", "Eid al-Ghadir"),
            ("2025-06-26", "Islamic New Year"),
            ("2025-07-05", "Ashura"),
            ("2025-07-21", "Great Feast"),
            ("2025-07-22", "Great Feast"),
            ("2025-07-23", "Great Feast"),
            ("2025-07-24", "Great Feast"),
            ("2025-08-02", "Yazidi Summer Festival"),
            ("2025-08-03", "Yazidi Summer Festival"),
            ("2025-09-04", "Prophet's Birthday"),
            ("2025-10-06", "Feast of the Assembly"),
            ("2025-10-07", "Feast of the Assembly"),
            ("2025-10-08", "Feast of the Assembly"),
            ("2025-10-09", "Feast of the Assembly"),
            ("2025-10-10", "Feast of the Assembly"),
            ("2025-10-11", "Feast of the Assembly"),
            ("2025-10-12", "Feast of the Assembly"),
            ("2025-10-13", "Feast of the Assembly"),
            ("2025-12-19", "Feast of Êzî"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import MAY, SEP, _get_nth_week_of_month, _timedelta
from holidays.constants import PUBLIC, WORKDAY
from holidays.groups import InternationalHolidays, MongolianCalendarHolidays
from holidays.holiday_base import HolidayBase


class Mongolia(HolidayBase, InternationalHolidays, MongolianCalendarHolidays):
    """Mongolia holidays.

    References:
        * [Law on Public holidays and days of observation](https://web.archive.org/web/20250327062440/https://legalinfo.mn/mn/detail/399)
        * [Labor Law](https://web.archive.org/web/20250421093230/https://legalinfo.mn/mn/detail?lawId=16230709635751)
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Mongolia>
        * [Mongolian lunar calendar](https://web.archive.org/web/20230412171012/https://www.math.mcgill.ca/gantumur/cal/index_mn.html)
        * <https://web.archive.org/web/20250613054645/https://www.timeanddate.com/holidays/mongolia/>
        * <https://web.archive.org/web/20250429110535/https://investmongolia.gov.mn/mongolia-at-a-glance/>
    """

    country = "MN"
    default_language = "mn"
    # %s (estimated).
    estimated_label = tr("%s (урьдчилсан)")
    start_year = 2004
    supported_categories = (PUBLIC, WORKDAY)
    supported_languages = ("en_US", "mn")

    def __init__(self, *args, **kwargs):
        InternationalHolidays.__init__(self)
        MongolianCalendarHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Шинэ жил"))

        # Lunar New Year.
        name = tr("Цагаан сар")
        self._add_tsagaan_sar(name)
        self._add_tsagaan_sar_day_2(name)
        # Expanded on November 28th, 2013.
        if self._year >= 2014:
            self._add_tsagaan_sar_day_3(name)

        # International Women's Day.
        self._add_womens_day(tr("Олон улсын эмэгтэйчүүдийн өдөр"))

        # Children's Day.
        self._add_childrens_day(tr("Хүүхдийн баяр"))

        # Established on December 20th, 2019.
        if self._year >= 2020:
            # The Buddha's Birthday.
            self._add_buddha_day(tr("Бурхан багшийн Их дүйчин өдөр"))

        # National Festival and People's Revolution Anniversary.
        name = tr("Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр")
        # Expanded to July 10 on July 7th, 2023.
        if self._year >= 2023:
            self._add_holiday_jul_10(name)
        self._add_holiday_jul_11(name)
        self._add_holiday_jul_12(name)
        self._add_holiday_jul_13(name)
        # Expanded to July 14-15 on July 1st, 2014.
        if self._year >= 2014:
            self._add_holiday_jul_14(name)
            self._add_holiday_jul_15(name)

        # Established on November 8th, 2012.
        if self._year >= 2012:
            # Genghis Khan's Birthday.
            self._add_genghis_khan_day(tr("Их Эзэн Чингис хааны өдөр"))

        # Repealed on November 8th, 2012.
        # Re-established again on November 18th, 2016.
        if self._year <= 2011 or self._year >= 2016:
            self._add_holiday_nov_26(
                # Republic Day.
                tr("Бүгд Найрамдах Улс тунхагласан өдөр")
                if self._year >= 2016
                # Republic Holiday.
                else tr("Бүгд Найрамдах Улс тунхагласны баяр")
            )

        # Established on August 16th, 2007.
        # Renamed and considered a Public Holiday on December 23rd, 2011.
        if self._year >= 2011:
            self._add_holiday_dec_29(
                # National Freedom and Independence Day.
                tr("Үндэсний эрх чөлөө, тусгаар тогтнолоо сэргээсний баярын өдөр")
            )

    def _populate_workday_holidays(self):
        # Constitution Day.
        self._add_holiday_jan_13(tr("Монгол Улсын Үндсэн хуулийн өдөр"))

        # Established on December 9th, 2004.
        if self._year >= 2005:
            # Patriots' Day.
            self._add_holiday_mar_1(tr("Эх орончдын өдөр"))

        # Renamed on February 9th, 2011.
        self._add_holiday_mar_18(
            # Military Day.
            tr("Монгол цэргийн өдөр")
            if self._year >= 2011
            # Armed Forces Day.
            else tr("Зэвсэгт хүчний өдөр")
        )

        # Health Protection Day.
        self._add_holiday_apr_7(tr("Эрүүл мэндийг хамгаалах өдөр"))

        # Intellectual Property Protection Day.
        self._add_holiday_apr_26(tr("Оюуны өмчийг хамгаалах өдөр"))

        # Family Day.
        self._add_holiday_may_15(tr("Гэр бүлийн өдөр"))

        # Established on June 11th, 2009.
        if self._year >= 2009:
            # National Flag Day.
            self._add_holiday_jul_10(tr("Монгол Улсын төрийн далбааны өдөр"))

        # Youth Day.
        self._add_holiday_aug_25(tr("Залуучуудын өдөр"))

        if self._year >= 2007:
            # New Harvest Days.
            self._add_multiday_holiday(self._add_holiday_sep_5(tr("Шинэ ургацын өдрүүд")), 45)

        # Memorial Day of Political Victims.
        self._add_holiday_sep_10(tr("Улс төрийн хэлмэгдэгсдийн дурсгалын өдөр"))

        # Established on July 2nd, 2021.
        if self._year >= 2021:
            # National Literary, Cultural and Book Days.
            name = tr("Үндэсний бичиг соёл, номын өдрүүд")
            sep_third_week_start = _get_nth_week_of_month(3, SEP, self._year)
            self._add_holiday(name, _timedelta(sep_third_week_start, +5))
            self._add_holiday(name, _timedelta(sep_third_week_start, +6))

            if self._year >= 2022:
                may_third_week_start = _get_nth_week_of_month(3, MAY, self._year)
                self._add_holiday(name, _timedelta(may_third_week_start, +5))
                self._add_holiday(name, _timedelta(may_third_week_start, +6))

        self._add_multiday_holiday(
            self._add_holiday(
                # Environmental Protection Days.
                tr("Байгаль орчныг хамгаалах өдрүүд"),
                _get_nth_week_of_month(4, SEP, self._year),
            ),
            6,
        )

        # Elders' Day.
        self._add_holiday_oct_1(tr("Ахмадын өдөр"))

        # Renamed on July 7th, 2021.
        self._add_holiday_oct_29(
            # Capital City Day.
            tr("Монгол Улсын нийслэл хотын өдөр")
            if self._year >= 2021
            # Capital Day.
            else tr("Монгол Улсын Нийслэлийн өдөр")
        )

        # Established on November 8th, 2012 after the old one was repealed.
        # Made a Public Holiday again on November 8th, 2016.
        if 2012 <= self._year <= 2015:
            # Republic Day.
            self._add_holiday_nov_26(tr("Бүгд Найрамдах Улс тунхагласан өдөр"))

        # Democracy and Human Rights Day.
        self._add_holiday_dec_10(tr("Ардчилал, хүний эрхийн өдөр"))

        # Established on August 16th, 2007.
        # Renamed and considered a Public Holiday on December 23rd, 2011.
        if 2007 <= self._year <= 2010:
            # National Freedom Day.
            self._add_holiday_dec_29(tr("Үндэсний эрх чөлөөний өдөр"))


class MN(Mongolia):
    pass


class MNG(Mongolia):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.paraguay import Paraguay
from tests.common import CommonCountryTests


class TestParaguay(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Paraguay, years_government=range(2010, 2026))

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_patriots_day(self):
        self.assertHolidayName(
            "Día de los Héroes de la Patria",
            (
                f"{year}-03-01"
                for year in self.full_range
                if year not in {2013, 2016, 2017, 2018, 2019, 2022, 2023, 2025}
            ),
            "2013-03-04",
            "2016-02-29",
            "2017-02-27",
            "2018-02-26",
            "2019-03-04",
            "2022-02-28",
            "2023-02-27",
            "2025-03-03",
        )

    def test_maundy_thursday(self):
        name = "Jueves Santo"
        self.assertHolidayName(
            name,
            "2019-04-18",
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Domingo de Resurrección"
        self.assertHolidayName(
            name,
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName(
            "Día de los Trabajadores", (f"{year}-05-01" for year in self.full_range)
        )

    def test_national_holiday(self):
        name = "Feriado Nacional"
        self.assertHolidayName(name, (f"{year}-05-14" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de la Independencia Nacional", (f"{year}-05-15" for year in self.full_range)
        )

    def test_chaco_armistice_day(self):
        self.assertHolidayName(
            "Día de la Paz del Chaco",
            (
                f"{year}-06-12"
                for year in self.full_range
                if year not in {2013, 2014, 2018, 2019, 2024, 2025}
            ),
            "2013-06-10",
            "2014-06-16",
            "2018-06-11",
            "2019-06-17",
            "2024-06-10",
            "2025-06-16",
        )

    def test_asuncion_foundations_day(self):
        self.assertHolidayName(
            "Día de la Fundación de Asunción",
            (
                f"{year}-08-15"
                for year in (*range(self.start_year, 2017), *range(2018, self.end_year))
            ),
            "2017-08-14",
        )

    def test_boqueron_battle_day(self):
        name = "Día de la Batalla de Boquerón"
        self.assertHolidayName(
            name,
            (
                f"{year}-09-29"
                for year in range(1995, self.end_year)
                if year not in {2015, 2016, 2017, 2021, 2022, 2024}
            ),
            "2015-09-28",
            "2016-10-03",
            "2017-10-02",
            "2021-09-27",
            "2022-10-03",
            "2024-09-30",
        )
        self.assertNoHolidayName(name, range(self.start_year, 1995))

    def test_caacupe_virgin_day(self):
        self.assertHolidayName(
            "Día de la Virgen de Caacupé", (f"{year}-12-08" for year in self.full_range)
        )

    def test_special_public_holidays(self):
        self.assertHolidayName(
            "Asueto adicional",
            "2007-01-29",
            "2009-09-10",
            "2010-06-14",
            "2011-04-19",
            "2011-05-14",
            "2011-05-16",
            "2013-08-14",
            "2015-07-10",
        )

    def test_special_government_holidays(self):
        self.assertGovernmentHolidayName(
            "Asueto de la Administración Pública",
            "2010-12-24",
            "2010-12-31",
            "2011-04-20",
            "2011-12-23",
            "2011-12-30",
            "2012-04-04",
            "2012-12-24",
            "2012-12-31",
            "2013-03-27",
            "2013-12-24",
            "2013-12-31",
            "2014-04-16",
            "2014-12-24",
            "2014-12-31",
            "2015-04-01",
            "2015-12-24",
            "2015-12-31",
            "2016-03-23",
            "2017-03-28",
            "2018-12-24",
            "2018-12-31",
            "2019-04-17",
            "2019-12-24",
            "2019-12-31",
            "2020-04-08",
            "2021-12-24",
            "2021-12-31",
            "2022-04-13",
            "2022-05-02",
            "2022-12-23",
            "2022-12-30",
            "2023-04-05",
            "2023-12-07",
            "2024-03-27",
            "2024-12-24",
            "2024-12-31",
            "2025-04-16",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-02-28", "Día de los Héroes de la Patria"),
            ("2022-04-13", "Asueto de la Administración Pública"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-04-17", "Domingo de Resurrección"),
            ("2022-05-01", "Día de los Trabajadores"),
            ("2022-05-02", "Asueto de la Administración Pública"),
            ("2022-05-14", "Feriado Nacional"),
            ("2022-05-15", "Día de la Independencia Nacional"),
            ("2022-06-12", "Día de la Paz del Chaco"),
            ("2022-08-15", "Día de la Fundación de Asunción"),
            ("2022-10-03", "Día de la Batalla de Boquerón"),
            ("2022-12-08", "Día de la Virgen de Caacupé"),
            ("2022-12-23", "Asueto de la Administración Pública"),
            ("2022-12-25", "Día de la Navidad"),
            ("2022-12-30", "Asueto de la Administración Pública"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-02-28", "Patriots Day"),
            ("2022-04-13", "Public sector holiday"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-05-01", "Workers' Day"),
            ("2022-05-02", "Public sector holiday"),
            ("2022-05-14", "National Holiday"),
            ("2022-05-15", "Independence Day"),
            ("2022-06-12", "Chaco Armistice Day"),
            ("2022-08-15", "Asuncion Foundation's Day"),
            ("2022-10-03", "Boqueron Battle Day"),
            ("2022-12-08", "Caacupe Virgin Day"),
            ("2022-12-23", "Public sector holiday"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-30", "Public sector holiday"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-02-28", "День національних героїв"),
            ("2022-04-13", "Вихідний державних установ"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-05-01", "День трудящих"),
            ("2022-05-02", "Вихідний державних установ"),
            ("2022-05-14", "Національне свято"),
            ("2022-05-15", "День незалежності"),
            ("2022-06-12", "День мирного договору в Чако"),
            ("2022-08-15", "День заснування Асунсьйона"),
            ("2022-10-03", "День битви за Бокерон"),
            ("2022-12-08", "День Богоматері Каакупе"),
            ("2022-12-23", "Вихідний державних установ"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-30", "Вихідний державних установ"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.north_korea import NorthKorea
from tests.common import CommonCountryTests


class TestNorthKorea(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1948, 2050)
        super().setUpClass(NorthKorea, years=years)

    def test_new_years_day(self):
        self.assertHolidayName("양력설", (f"{year}-01-01" for year in range(1948, 2050)))

    def test_korean_new_years_day(self):
        name = "설명절"
        self.assertHolidayName(
            name,
            "2020-01-25",
            "2021-02-12",
            "2022-02-01",
            "2023-01-22",
            "2024-02-10",
            "2025-01-29",
        )
        self.assertHolidayName(name, range(1948, 1968), range(1989, 2050))
        self.assertNoHolidayName(name, range(1968, 1989))

    def test_daeboreum_day(self):
        name = "대보름"
        self.assertHolidayName(
            name,
            "2020-02-08",
            "2021-02-26",
            "2022-02-15",
            "2023-02-05",
            "2024-02-24",
            "2025-02-12",
        )
        self.assertHolidayName(name, range(2003, 2050))
        self.assertNoHolidayName(name, range(1948, 2003))

    def test_founding_day_of_peoples_army(self):
        name = "조선인민군창건일"
        self.assertHolidayName(
            name, (f"{year}-02-08" for year in (*range(1948, 1978), *range(2018, 2050)))
        )
        self.assertNoHolidayName(name, range(1978, 2018))

    def test_day_of_shining_star(self):
        name_1975 = "김정일의 생일"
        self.assertHolidayName(
            name_1975,
            (f"{year}-02-16" for year in range(1975, 2012)),
            (f"{year}-02-17" for year in range(1986, 2012)),
        )
        self.assertNoHolidayName(name_1975, range(1948, 1975), range(2012, 2050))

        name_2012 = "광명성절"
        self.assertHolidayName(
            name_2012,
            (f"{year}-02-16" for year in range(2012, 2050)),
            (f"{year}-02-17" for year in range(2012, 2050)),
        )
        self.assertNoHolidayName(name_2012, range(1948, 2012))

    def test_womens_day(self):
        self.assertHolidayName("국제부녀절", (f"{year}-03-08" for year in range(1948, 2050)))

    def test_cheongmyeong_festival(self):
        name = "청명"
        self.assertHolidayName(
            name,
            "2020-04-04",
            "2021-04-04",
            "2022-04-05",
            "2023-04-05",
            "2024-04-04",
            "2025-04-04",
        )
        self.assertHolidayName(name, range(2012, 2050))
        self.assertNoHolidayName(name, range(1948, 2012))

    def test_hanshi_festival(self):
        name = "한식"
        self.assertHolidayName(
            name,
            "1963-04-06",
            "1964-04-05",
            "1965-04-06",
            "1966-04-06",
            "1967-04-06",
        )
        self.assertHolidayName(name, range(1948, 1968))
        self.assertNoHolidayName(name, range(1968, 2050))

    def test_day_of_sun(self):
        name_1962 = "김일성의 생일"
        self.assertHolidayName(
            name_1962, "1962-04-15", (f"{year}-04-15" for year in range(1968, 1998))
        )
        self.assertNoHolidayName(
            name_1962, range(1948, 1962), range(1963, 1968), range(1998, 2050)
        )

        name_1998 = "태양절"
        self.assertHolidayName(
            name_1998,
            (f"{year}-04-15" for year in range(1998, 2050)),
            (f"{year}-04-16" for year in range(1998, 2050)),
        )
        self.assertNoHolidayName(name_1998, range(1948, 1998))

    def test_army_day(self):
        name_1978 = "건군절"
        self.assertHolidayName(name_1978, (f"{year}-04-25" for year in range(1978, 2018)))
        self.assertNoHolidayName(name_1978, range(1948, 1978), range(2018, 2050))

        name_2018 = "조선인민혁명군 창건일"
        self.assertHolidayName(name_2018, (f"{year}-04-25" for year in range(2018, 2050)))
        self.assertNoHolidayName(name_2018, range(1948, 2018))

    def test_workers_day(self):
        self.assertHolidayName(
            "전세계근로자들의 국제적명절", (f"{year}-05-01" for year in range(1948, 2050))
        )

    def test_dano_festival(self):
        name = "단오"
        self.assertHolidayName(
            name,
            "1963-06-25",
            "1964-06-14",
            "1965-06-04",
            "1966-06-23",
            "1967-06-12",
        )
        self.assertHolidayName(name, range(1948, 1968))
        self.assertNoHolidayName(name, range(1968, 2050))

    def test_childrens_union_day(self):
        self.assertHolidayName(
            "조선소년단 창립절", (f"{year}-06-06" for year in range(1948, 2050))
        )

    def test_victory_day(self):
        name = "조국해방전쟁승리기념일"
        self.assertHolidayName(name, (f"{year}-07-27" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, range(1948, 1996))

    def test_liberation_day(self):
        self.assertHolidayName("조국해방절", (f"{year}-08-15" for year in range(1948, 2050)))

    def test_chuseok(self):
        name = "추석"
        self.assertHolidayName(
            name,
            "2020-10-01",
            "2021-09-21",
            "2022-09-10",
            "2023-09-29",
            "2024-09-17",
            "2025-10-06",
        )
        self.assertHolidayName(name, range(1948, 1967), range(1989, 2050))
        self.assertNoHolidayName(name, range(1967, 1989))

    def test_day_of_songun(self):
        name = "선군절"
        self.assertHolidayName(name, (f"{year}-08-25" for year in range(2013, 2050)))
        self.assertNoHolidayName(name, range(1948, 2013))

    def test_youth_day(self):
        self.assertHolidayName("청년절", (f"{year}-08-28" for year in range(1948, 2050)))

    def test_foundation_day(self):
        self.assertHolidayName(
            "조선민주주의인민공화국창건일", (f"{year}-09-09" for year in range(1948, 2050))
        )

    def test_party_foundation_day(self):
        self.assertHolidayName("조선로동당창건일", (f"{year}-10-10" for year in range(1948, 2050)))

    def test_mothers_day(self):
        name = "어머니날"
        self.assertHolidayName(name, (f"{year}-11-16" for year in range(2012, 2050)))
        self.assertNoHolidayName(name, range(1948, 2012))

    def test_constitution_day(self):
        name = "사회주의헌법절"
        self.assertHolidayName(name, (f"{year}-12-27" for year in range(1972, 2050)))
        self.assertNoHolidayName(name, range(1948, 1972))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "양력설"),
            ("2024-02-08", "조선인민군창건일"),
            ("2024-02-10", "설명절"),
            ("2024-02-16", "광명성절"),
            ("2024-02-17", "광명성절"),
            ("2024-02-24", "대보름"),
            ("2024-03-08", "국제부녀절"),
            ("2024-04-04", "청명"),
            ("2024-04-15", "태양절"),
            ("2024-04-16", "태양절"),
            ("2024-04-25", "조선인민혁명군 창건일"),
            ("2024-05-01", "전세계근로자들의 국제적명절"),
            ("2024-06-06", "조선소년단 창립절"),
            ("2024-07-27", "조국해방전쟁승리기념일"),
            ("2024-08-15", "조국해방절"),
            ("2024-08-25", "선군절"),
            ("2024-08-28", "청년절"),
            ("2024-09-09", "조선민주주의인민공화국창건일"),
            ("2024-09-17", "추석"),
            ("2024-10-10", "조선로동당창건일"),
            ("2024-11-16", "어머니날"),
            ("2024-12-27", "사회주의헌법절"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-02-08", "Founding Day of the Korean People's Army"),
            ("2024-02-10", "Korean New Year"),
            ("2024-02-16", "Day of the Shining Star"),
            ("2024-02-17", "Day of the Shining Star"),
            ("2024-02-24", "Daeboreum"),
            ("2024-03-08", "International Women's Day"),
            ("2024-04-04", "Cheongmyeong Festival"),
            ("2024-04-15", "Day of the Sun"),
            ("2024-04-16", "Day of the Sun"),
            ("2024-04-25", "Founding Day of the Korean People's Revolutionary Army"),
            ("2024-05-01", "International Workers' Day"),
            ("2024-06-06", "Foundation Day of the Korean Children's Union"),
            ("2024-07-27", "Day of Victory in the Great Fatherland Liberation War"),
            ("2024-08-15", "Liberation Day"),
            ("2024-08-25", "Day of Songun"),
            ("2024-08-28", "Youth Day"),
            ("2024-09-09", "Founding Day of the DPRK"),
            ("2024-09-17", "Chuseok"),
            ("2024-10-10", "Foundation Day of the Workers' Party of Korea"),
            ("2024-11-16", "Mother's Day"),
            ("2024-12-27", "Socialist Constitution Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.bolivia import Bolivia
from tests.common import CommonCountryTests


class TestBolivia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        cls.full_range = range(1980, 2050)
        super().setUpClass(Bolivia, years_non_observed=range(2000, 2024))

    def test_new_years_day(self):
        name = "Año Nuevo"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_plurinational_state_foundation_day(self):
        name = "Día de la Creación del Estado Plurinacional de Bolivia"
        self.assertHolidayName(name, (f"{year}-01-22" for year in range(2010, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2010))
        obs_dts = (
            "2012-01-23",
            "2017-01-23",
            "2023-01-23",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_carnival(self):
        name = "Carnaval"
        self.assertHolidayName(
            name,
            "2015-02-16",
            "2015-02-17",
            "2016-02-08",
            "2016-02-09",
            "2017-02-27",
            "2017-02-28",
            "2018-02-12",
            "2018-02-13",
            "2019-03-04",
            "2019-03-05",
            "2020-02-24",
            "2020-02-25",
            "2021-02-15",
            "2021-02-16",
            "2022-02-28",
            "2022-03-01",
            "2023-02-20",
            "2023-02-21",
        )
        self.assertHolidayNameCount(name, 2, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2015-04-03",
            "2016-03-25",
            "2017-04-14",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Día del Trabajo"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2005-05-02",
            "2011-05-02",
            "2012-04-30",
            "2014-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_corpus_christi(self):
        name = "Corpus Christi"
        self.assertHolidayName(
            name,
            "2015-06-04",
            "2016-05-26",
            "2017-06-15",
            "2018-05-31",
            "2019-06-20",
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
        )
        self.assertHolidayName(name, self.full_range)

    def test_andean_new_year(self):
        name = "Año Nuevo Aymara Amazónico"
        self.assertHolidayName(name, (f"{year}-06-21" for year in range(2009, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2009))
        obs_dts = (
            "2009-06-22",
            "2015-06-22",
            "2020-06-22",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Día de la Independencia de Bolivia"
        self.assertHolidayName(name, (f"{year}-08-06" for year in self.full_range))
        obs_dts = (
            "2000-08-07",
            "2006-08-07",
            "2017-08-07",
            "2023-08-07",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_dignity_day(self):
        name = "Día de la Dignidad Nacional"
        self.assertHolidayName(name, (f"{year}-10-17" for year in range(2020, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2020))

    def test_all_saints_day(self):
        name = "Día de Todos los Santos"
        self.assertHolidayName(name, (f"{year}-11-01" for year in range(1985, 1989)))
        self.assertNoHolidayName(name, range(self.start_year, 1985), range(1989, self.end_year))

    def test_all_souls_day(self):
        name = "Día de Todos los Difuntos"
        self.assertHolidayName(name, (f"{year}-11-02" for year in range(1989, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1989))
        obs_dts = (
            "2003-11-03",
            "2008-11-03",
            "2014-11-03",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNoHolidayName(f"{name} (observado)", range(2016, self.end_year))

    def test_christmas_day(self):
        name = "Navidad"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2005-12-26",
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_beni_day(self):
        name = "Día del departamento de Beni"
        self.assertNoHolidayName(name, self.full_range)
        self.assertSubdivBHolidayName(name, (f"{year}-11-18" for year in self.full_range))

    def test_cochabamba_day(self):
        name = "Día del departamento de Cochabamba"
        self.assertNoHolidayName(name, self.full_range)
        self.assertSubdivCHolidayName(name, (f"{year}-09-14" for year in self.full_range))

    def test_chuquisaca_day(self):
        name = "Día del departamento de Chuquisaca"
        self.assertNoHolidayName(name, self.full_range)
        self.assertSubdivHHolidayName(name, (f"{year}-05-25" for year in self.full_range))

    def test_la_paz_day(self):
        name = "Día del departamento de La Paz"
        self.assertNoHolidayName(name, self.full_range)
        self.assertSubdivLHolidayName(name, (f"{year}-07-16" for year in self.full_range))

    def test_pando_day(self):
        name = "Día del departamento de Pando"
        self.assertNoHolidayName(name, self.full_range)
        self.assertSubdivNHolidayName(name, (f"{year}-10-11" for year in self.full_range))

    def test_potosi_day(self):
        name = "Día del departamento de Potosí"
        self.assertNoHolidayName(name, self.full_range)
        self.assertSubdivPHolidayName(name, (f"{year}-11-10" for year in self.full_range))

    def test_carnival_in_oruro(self):
        name = "Carnaval de Oruro"
        self.assertNoHolidayName(name, self.full_range)
        self.assertSubdivOHolidayName(
            name,
            "2015-02-13",
            "2016-02-05",
            "2017-02-24",
            "2018-02-09",
            "2019-03-01",
            "2020-02-21",
            "2021-02-12",
            "2022-02-25",
            "2023-02-17",
        )
        self.assertSubdivOHolidayName(name, self.full_range)

    def test_santa_cruz_day(self):
        name = "Día del departamento de Santa Cruz"
        self.assertNoHolidayName(name, self.full_range)
        self.assertSubdivSHolidayName(name, (f"{year}-09-24" for year in self.full_range))

    def test_la_tablada(self):
        name = "La Tablada"
        self.assertNoHolidayName(name, self.full_range)
        self.assertSubdivTHolidayName(name, (f"{year}-04-15" for year in self.full_range))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Año Nuevo"),
            ("2023-01-02", "Año Nuevo (observado)"),
            ("2023-01-22", "Día de la Creación del Estado Plurinacional de Bolivia"),
            ("2023-01-23", "Día de la Creación del Estado Plurinacional de Bolivia (observado)"),
            ("2023-02-17", "Carnaval de Oruro"),
            ("2023-02-20", "Carnaval"),
            ("2023-02-21", "Carnaval"),
            ("2023-04-07", "Viernes Santo"),
            ("2023-04-15", "La Tablada"),
            ("2023-05-01", "Día del Trabajo"),
            ("2023-05-25", "Día del departamento de Chuquisaca"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-06-21", "Año Nuevo Aymara Amazónico"),
            ("2023-07-16", "Día del departamento de La Paz"),
            ("2023-08-06", "Día de la Independencia de Bolivia"),
            ("2023-08-07", "Día de la Independencia de Bolivia (observado)"),
            ("2023-09-14", "Día del departamento de Cochabamba"),
            ("2023-09-24", "Día del departamento de Santa Cruz"),
            ("2023-10-11", "Día del departamento de Pando"),
            ("2023-10-17", "Día de la Dignidad Nacional"),
            ("2023-11-02", "Día de Todos los Difuntos"),
            ("2023-11-10", "Día del departamento de Potosí"),
            ("2023-11-18", "Día del departamento de Beni"),
            ("2023-12-25", "Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-01-22", "Plurinational State Foundation Day"),
            ("2023-01-23", "Plurinational State Foundation Day (observed)"),
            ("2023-02-17", "Carnival in Oruro"),
            ("2023-02-20", "Carnival"),
            ("2023-02-21", "Carnival"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-15", "La Tablada"),
            ("2023-05-01", "Labor Day"),
            ("2023-05-25", "Chuquisaca Day"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-06-21", "Aymara New Year"),
            ("2023-07-16", "La Paz Day"),
            ("2023-08-06", "Independence Day"),
            ("2023-08-07", "Independence Day (observed)"),
            ("2023-09-14", "Cochabamba Day"),
            ("2023-09-24", "Santa Cruz Day"),
            ("2023-10-11", "Pando Day"),
            ("2023-10-17", "National Dignity Day"),
            ("2023-11-02", "All Souls' Day"),
            ("2023-11-10", "Potosí Day"),
            ("2023-11-18", "Beni Day"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-01-02", "Новий рік (вихідний)"),
            ("2023-01-22", "День створення Багатонаціональної Держави Болівія"),
            ("2023-01-23", "День створення Багатонаціональної Держави Болівія (вихідний)"),
            ("2023-02-17", "Карнавал Оруро"),
            ("2023-02-20", "Карнавал"),
            ("2023-02-21", "Карнавал"),
            ("2023-04-07", "Страсна пʼятниця"),
            ("2023-04-15", "Ла Таблада"),
            ("2023-05-01", "День праці"),
            ("2023-05-25", "День департаменту Чукісака"),
            ("2023-06-08", "Свято Тіла і Крові Христових"),
            ("2023-06-21", "Новий рік Аймара"),
            ("2023-07-16", "День департаменту Ла-Пас"),
            ("2023-08-06", "День незалежності Болівії"),
            ("2023-08-07", "День незалежності Болівії (вихідний)"),
            ("2023-09-14", "День департаменту Кочабамба"),
            ("2023-09-24", "День департаменту Санта-Круз"),
            ("2023-10-11", "День департаменту Пандо"),
            ("2023-10-17", "День національної гідності"),
            ("2023-11-02", "День усіх померлих"),
            ("2023-11-10", "День департаменту Потосі"),
            ("2023-11-18", "День департаменту Бені"),
            ("2023-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.constants import GOVERNMENT, PUBLIC, WORKDAY
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class PitcairnIslands(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Pitcairn Islands holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Pitcairn_Islands>
        * [Public Holidays & Commemoration Days (2016)](https://web.archive.org/web/20161112174923/https://www.government.pn/policies/GPI%20-%20Approved%20Public%20Holidays%20&%20Commemoration%20Days.pdf)
        * [Public Holidays & Commemoration Days (2024)](https://web.archive.org/web/20250517050245/https://static1.squarespace.com/static/6526ff6fef608a3828c13d05/t/6673d53b1c0e2f660cc31848/1718867262145/GPI_Policy_Public_Holidays_&_Commemoration_Days_June_2024.pdf)
        * [The Laws of Pitcairn, Henderson, Ducie and Oeno Islands - Volume I](https://web.archive.org/web/20240418043000/https://static1.squarespace.com/static/6526ff6fef608a3828c13d05/t/65585fca2ce3972fb3bc8da3/1700290563094/Revised+Laws+of+Pitcairn,+Henderson,+Ducie+and+Oeno+Islands,+2017+Rev.+Ed.+-+Volume+1.pdf)
        * [CIA The World Factbook](https://web.archive.org/web/20211217233431/https://www.cia.gov/the-world-factbook/countries/pitcairn-islands/)
    """

    country = "PN"
    supported_categories = (GOVERNMENT, PUBLIC, WORKDAY)
    # First available online source.
    start_year = 2016

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day("New Year's Day")

        # Bounty Day.
        self._add_holiday_jan_23("Bounty Day")

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        if self._year >= 2023:
            # King's Birthday.
            self._add_holiday_1st_mon_of_jun("King's Birthday")
        else:
            # Queen's Birthday.
            self._add_holiday_2nd_sat_of_jun("Queen's Birthday")

        # Pitcairn Day.
        self._add_holiday_jul_2("Pitcairn Day")

        # Christmas Day.
        self._add_christmas_day("Christmas Day")

        # Boxing Day.
        self._add_christmas_day_two("Boxing Day")

    def _populate_government_holidays(self):
        # New Year's Day.
        self._add_new_years_day_two("New Year's Day")

    def _populate_workday_holidays(self):
        # ANZAC Day.
        self._add_anzac_day("ANZAC Day")

        # Remembrance Day.
        self._add_remembrance_day("Remembrance Day")


class PN(PitcairnIslands):
    pass


class PCN(PitcairnIslands):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import FEB, MAR, APR, MAY, AUG, OCT, DEC, SUN
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Venezuela(HolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Venezuela holidays.

    References:
        * <https://web.archive.org/web/20250317124608/https://dias-festivos.eu/dias-festivos/venezuela/>
        * <https://web.archive.org/web/20260104061304/https://ecoportal50a.tripod.com/directur-fiestas.htm>
        * [1909 Aug 5](https://web.archive.org/web/20241012151721/https://www.guao.org/sites/default/files/efemerides/69.Ley%20fiestas%20nacionales%201909.pdf)
        * [1918 May 19](https://web.archive.org/web/20250429081541/https://www.guao.org/sites/default/files/efemerides/70.%20Ley%20de%20fiestas%20nacionales%201918.pdf)
        * [1921 Jun 11](https://web.archive.org/web/20241012160001/https://www.guao.org/sites/default/files/efemerides/37.LEYES_Y_DECRETOS_1921_Día_de_la_raza.PDF)
        * [1945 Apr 30](https://web.archive.org/web/20250428122616/https://venezuelaenretrospectiva.wordpress.com/2016/05/01/1o-de-mayo-dia-del-trabajador-venezolano/)
        * [1971 Jun 22](https://web.archive.org/web/20240623070739/https://docs.venezuela.justia.com/federales/leyes/ley-de-fiestas-nacionales.pdf)
        * [2002 Oct 10](https://web.archive.org/web/20250214140239/https://www.acnur.org/fileadmin/Documentos/BDL/2008/6635.pdf)
        * [2012 May 7](https://web.archive.org/web/20250418204844/https://oig.cepal.org/sites/default/files/2012_leyorgtrabajo_ven.pdf)
        * [Family Day](https://web.archive.org/web/20240516041705/https://www.globovision.com/nacional/21451/ejecutivo-decreta-feriado-el-15-de-mayo-por-dia-de-la-familia)
    """

    country = "VE"
    default_language = "es"
    supported_languages = ("en_US", "es", "uk")
    # Ley de Fiestas Nacionales de 5 de agosto de 1909.
    start_year = 1910
    weekend = {SUN}

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, VenezuelaStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Año Nuevo"))

        # Carnival Monday.
        self._add_carnival_monday(tr("Lunes de Carnaval"))

        # Carnival Tuesday.
        self._add_carnival_tuesday(tr("Martes de Carnaval"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Jueves Santo"))

        # Good Friday.
        self._add_good_friday(tr("Viernes Santo"))

        # Declaration of Independence.
        self._add_holiday_apr_19(tr("Declaración de la Independencia"))

        # Added via Decreto 113 de 30 de abril de 1945.
        if self._year >= 1945:
            # International Worker's Day.
            self._add_labor_day(tr("Dia Mundial del Trabajador"))

        # Added in 2024, confirmed to be observed in 2025.
        if self._year >= 2024:
            # Family Day.
            self._add_holiday_may_15(tr("Día de la Familia"))

        # Added via via Ley de Fiestas Nacionales de 5 de agosto de 1909.
        # Removed via Ley de Fiestas Nacionales de 19 de mayo de 1918.
        # Readded via Ley de Fiestas Nacionales de 22 de junio de 1971.
        if self._year <= 1917 or self._year >= 1971:
            # Battle of Carabobo.
            self._add_holiday_jun_24(tr("Batalla de Carabobo"))

        # Independence Day.
        self._add_holiday_jul_5(tr("Día de la Independencia"))

        # Added via Ley de Fiestas Nacionales de 19 de mayo de 1918.
        if self._year >= 1918:
            # Birthday of Simon Bolivar.
            self._add_holiday_jul_24(tr("Natalicio de Simón Bolívar"))

        # Added via Ley de Fiestas Nacionales de 11 de junio de 1921.
        # Name changed via Decreto N° 2.028, 10 de octubre de 2002.
        if self._year >= 1921:
            self._add_columbus_day(
                # Day of Indigenous Resistance.
                tr("Día de la Resistencia Indígena")
                if self._year >= 2002
                # Columbus Day.
                else tr("Día de la Raza")
            )

        # Added via Ley de Fiestas Nacionales de 5 de agosto de 1909.
        # Removed via Ley de Fiestas Nacionales de 19 de mayo de 1918.
        if self._year <= 1917:
            # Feast of Saint Simon the Zealot.
            self._add_holiday_oct_28(tr("Fiesta de San Simón Apóstol"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Nochebuena"))

        # Christmas Day.
        self._add_christmas_day(tr("Día de Navidad"))

        # New Year's Eve.
        self._add_new_years_eve(tr("Fiesta de Fin de Año"))


class VE(Venezuela):
    pass


class VEN(Venezuela):
    pass


class VenezuelaStaticHolidays:
    """Venezuela special holidays.

    References:
        * [February 2nd, 2009](https://web.archive.org/web/20260104075141/https://venezuelanalysis.com/news/4164/)
        * [March 29th-31st, 2010](https://web.archive.org/web/20241013111835/https://accesoalajusticia.org/wp-content/uploads/2017/08/SPA-N%C2%BA-869-01-08-2017.pdf)
        * [December 8th, 2013](https://web.archive.org/web/20230129224800/https://www.bbc.com/mundo/ultimas_noticias/2013/11/131105_ultnot_venezuela_dia_lealtad_chavez)
        * [February 27th-28th, 2014](https://web.archive.org/web/20201111233441/https://www.economist.com/the-americas/2014/03/01/towards-the-brink)
        * [March 21st-23rd, 2016](https://web.archive.org/web/20250803061045/https://kpmg.com/ve/es/home/insights/2016/03/sintesis-legal--11-2016.html)
        * [All Fridays in April & May, 2016](https://web.archive.org/web/20260104055041/https://www.abs-cbn.com/world/v1/04/08/16/venezuela-decrees-fridays-a-holiday-to-ease-energy-crisis)
        * [April 18th, 2016](https://web.archive.org/web/20160422035408/https://www.arabnews.com/world/news/910856)
        * [April 10th-12th, 2017](https://web.archive.org/web/20260104080641/https://interjuris.com/dias-no-laborables-sector-publico/)
        * [August 20th, 2018](https://web.archive.org/web/20260104074101/https://www.afaca.com.ve/wp-content/uploads/2018/08/Gaceta-oficial-iva-a-16.pdf)
        * [February 28th-March 1st, 2019](https://web.archive.org/web/20260104073806/https://interjuris.com/non-working-days/)
        * [March 11th-12th, 2019](https://web.archive.org/web/20260104065639/https://www.aa.com.tr/en/energy/electricity/venezuela-extends-public-holiday-over-power-outage/23799)
        * [March 26th, 2019](https://web.archive.org/web/20260104065935/https://www.latimes.com/espanol/noticas-mas/articulo/2019-03-26/efe-3935909-15268311-20190326)
        * [October 19th-20th, 2025](https://web.archive.org/web/20260104073119/https://finanzasdigital.com/gaceta-oficial-6933-dias-no-laborables-19-20-octubre-2025/)
    """

    # National Holidays.
    national_holidays = tr("Fiesta Nacional")

    # Canonization of José Gregorio Hernández and Mother Carmen Rendiles.
    canonization = tr("Canonización de José Gregorio Hernández y la Madre Carmen Rendiles")

    special_public_holidays = {
        # 10th Anniversary of the Bolivarian Revolution.
        2009: (FEB, 2, tr("10.º aniversario de la Revolución Bolivariana")),
        2010: (
            (MAR, 29, national_holidays),
            (MAR, 30, national_holidays),
            (MAR, 31, national_holidays),
        ),
        2013: (
            DEC,
            8,
            # Day of Loyalty and Love for Supreme Commander Hugo Chávez and the Fatherland.
            tr("Día de la Lealtad y el Amor al Comandante Supremo Hugo Chávez y a la Patria"),
        ),
        2014: (
            (FEB, 27, national_holidays),
            (FEB, 28, national_holidays),
        ),
        2016: (
            (MAR, 21, national_holidays),
            (MAR, 22, national_holidays),
            (MAR, 23, national_holidays),
            (APR, 8, national_holidays),
            (APR, 15, national_holidays),
            (APR, 18, national_holidays),
            (APR, 22, national_holidays),
            (APR, 29, national_holidays),
            (MAY, 6, national_holidays),
            (MAY, 13, national_holidays),
            (MAY, 20, national_holidays),
            (MAY, 27, national_holidays),
        ),
        2017: (
            (APR, 10, national_holidays),
            (APR, 11, national_holidays),
            (APR, 12, national_holidays),
        ),
        2018: (AUG, 20, national_holidays),
        2019: (
            (FEB, 28, national_holidays),
            (MAR, 1, national_holidays),
            (MAR, 11, national_holidays),
            (MAR, 12, national_holidays),
            (MAR, 26, national_holidays),
        ),
        2025: (
            (OCT, 19, canonization),
            (OCT, 20, canonization),
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import WORKDAY
from holidays.countries.slovenia import Slovenia
from tests.common import CommonCountryTests


class TestSlovenia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Slovenia)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Slovenia(categories=WORKDAY, years=range(self.start_year, 2005)))

    def test_special_holidays(self):
        self.assertHoliday(
            # Solidarity Day.
            "2023-08-14",
        )

    def test_new_years_day(self):
        name = "novo leto"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertHolidayName(
            name,
            (
                f"{year}-01-02"
                for year in (*range(self.start_year, 2013), *range(2017, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, (f"{year}-01-02" for year in range(2013, 2017)))

    def test_preserens_day_the_slovenian_cultural_holiday(self):
        self.assertHolidayName(
            "Prešernov dan, slovenski kulturni praznik",
            (f"{year}-02-08" for year in self.full_range),
        )

    def test_easter_sunday(self):
        name = "velikonočna nedelja"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "velikonočni ponedeljek"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_day_of_uprising_against_occupation(self):
        self.assertHolidayName(
            "dan upora proti okupatorju", (f"{year}-04-27" for year in self.full_range)
        )

    def test_labor_day(self):
        self.assertHolidayName(
            "praznik dela",
            (f"{year}-05-01" for year in self.full_range),
            (f"{year}-05-02" for year in self.full_range),
        )

    def test_whit_sunday(self):
        name = "binkoštna nedelja"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, self.full_range)

    def test_statehood_day(self):
        self.assertHolidayName("dan državnosti", (f"{year}-06-25" for year in self.full_range))

    def test_assumption_day(self):
        self.assertHolidayName(
            "Marijino vnebovzetje", (f"{year}-08-15" for year in self.full_range)
        )

    def test_reformation_day(self):
        self.assertHolidayName("dan reformacije", (f"{year}-10-31" for year in self.full_range))

    def test_day_of_remembrance_for_the_dead(self):
        self.assertHolidayName(
            "dan spomina na mrtve", (f"{year}-11-01" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName("božič", (f"{year}-12-25" for year in self.full_range))

    def test_independence_and_unity_day(self):
        name_1992 = "dan samostojnosti"
        name_2005 = "dan samostojnosti in enotnosti"
        self.assertHolidayName(
            name_1992, (f"{year}-12-26" for year in range(self.start_year, 2005))
        )
        self.assertHolidayName(name_2005, (f"{year}-12-26" for year in range(2005, self.end_year)))
        self.assertNoHolidayName(name_1992, range(2005, self.end_year))
        self.assertNoHolidayName(name_2005, range(self.start_year, 2005))

    def test_primoz_trubar_day(self):
        name = "dan Primoža Trubarja"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-06-08" for year in range(2011, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2011))

    def test_unification_of_prekmurje_slovenes_with_the_mother_nation(self):
        name = "združitev prekmurskih Slovencev z matičnim narodom"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-08-17" for year in range(2006, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2006))

    def test_integration_of_primorska_into_the_homeland(self):
        name_2006 = "vrnitev Primorske k matični domovini"
        name_2025 = "priključitev Primorske k matični domovini"
        self.assertNoHolidayName(name_2006)
        self.assertNoHolidayName(name_2025)
        self.assertWorkdayHolidayName(name_2006, (f"{year}-09-15" for year in range(2006, 2025)))
        self.assertWorkdayHolidayName(
            name_2025, (f"{year}-09-15" for year in range(2025, self.end_year))
        )
        self.assertNoWorkdayHolidayName(
            name_2006, range(self.start_year, 2006), range(2025, self.end_year)
        )
        self.assertNoWorkdayHolidayName(name_2025, range(self.start_year, 2025))

    def test_slovenian_sports_day(self):
        name = "dan slovenskega športa"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-09-23" for year in range(2020, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2020))

    def test_sovereignty_day(self):
        name = "dan suverenosti"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-10-25" for year in range(2015, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2015))

    def test_rudolf_maister_day(self):
        name = "dan Rudolfa Maistra"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-11-23" for year in range(2005, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2005))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "novo leto"),
            ("2022-01-02", "novo leto"),
            ("2022-02-08", "Prešernov dan, slovenski kulturni praznik"),
            ("2022-04-17", "velikonočna nedelja"),
            ("2022-04-18", "velikonočni ponedeljek"),
            ("2022-04-27", "dan upora proti okupatorju"),
            ("2022-05-01", "praznik dela"),
            ("2022-05-02", "praznik dela"),
            ("2022-06-05", "binkoštna nedelja"),
            ("2022-06-08", "dan Primoža Trubarja"),
            ("2022-06-25", "dan državnosti"),
            ("2022-08-15", "Marijino vnebovzetje"),
            ("2022-08-17", "združitev prekmurskih Slovencev z matičnim narodom"),
            ("2022-09-15", "vrnitev Primorske k matični domovini"),
            ("2022-09-23", "dan slovenskega športa"),
            ("2022-10-25", "dan suverenosti"),
            ("2022-10-31", "dan reformacije"),
            ("2022-11-01", "dan spomina na mrtve"),
            ("2022-11-23", "dan Rudolfa Maistra"),
            ("2022-12-25", "božič"),
            ("2022-12-26", "dan samostojnosti in enotnosti"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "New Year's Day"),
            ("2022-02-08", "Prešeren's Day, the Slovenian Cultural Holiday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-27", "Day of Uprising Against Occupation"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Labor Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-08", "Primož Trubar Day"),
            ("2022-06-25", "Statehood Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-08-17", "Unification of Prekmurje Slovenes with the Mother Nation"),
            ("2022-09-15", "Return of Primorska into the Homeland"),
            ("2022-09-23", "Slovenian Sport's Day"),
            ("2022-10-25", "Sovereignty Day"),
            ("2022-10-31", "Reformation Day"),
            ("2022-11-01", "Day of Remembrance for the Dead"),
            ("2022-11-23", "Rudolf Maister Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Independence and Unity Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-02", "Новий рік"),
            ("2022-02-08", "День Прешерена, свято словенської культури"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-04-27", "День спротиву окупантам"),
            ("2022-05-01", "День праці"),
            ("2022-05-02", "День праці"),
            ("2022-06-05", "Трійця"),
            ("2022-06-08", "День Приможа Трубара"),
            ("2022-06-25", "День державності"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-08-17", "Обʼєднання прекмурських словенців з материнською нацією"),
            ("2022-09-15", "Повернення Словенського Приморʼя до батьківщини"),
            ("2022-09-23", "День словенського спорту"),
            ("2022-10-25", "День суверенітету"),
            ("2022-10-31", "День Реформації"),
            ("2022-11-01", "День памʼяті померлих"),
            ("2022-11-23", "День Рудольфа Майстера"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "День незалежності та єднання"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JUL
from holidays.constants import BANK, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Panama(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Panama holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Panama>
        * [Labor Code 1947](https://web.archive.org/web/20250429081529/https://docs.panama.justia.com/federales/leyes/67-de-1947-nov-26-1947.pdf)
        * [Cabinet Decree #347 of 1969](https://web.archive.org/web/20250429081519/https://docs.panama.justia.com/federales/decretos-de-gabinete/decreto-de-gabinete-347-de-1969-nov-14-1969.pdf)
        * [Labor Code 1971](https://web.archive.org/web/20240420050417/https://www.mitradel.gob.pa/wp-content/uploads/2016/12/código-detrabajo.pdf)
        * [Law #4 of Jun 25, 1990](https://web.archive.org/web/20250429081538/https://s3-legispan.asamblea.gob.pa/legispan/NORMAS/1990/1990/LEY/Administrador%20Legispan_21570_1990_7_2_ASAMBLEA%20LEGISLATIVA_4.pdf)
        * [Law #55 of Nov 7, 2001](https://web.archive.org/web/20230401210617/https://docs.panama.justia.com/federales/leyes/55-de-2001-nov-14-2001.pdf)
    """

    country = "PA"
    default_language = "es"
    # %s (observed).
    observed_label = tr("%s (puente)")
    supported_categories = (BANK, PUBLIC)
    supported_languages = ("en_US", "es", "uk")
    start_year = 1948

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, PanamaStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        kwargs.setdefault("observed_since", 1972)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("Año Nuevo")))

        if self._year >= 1972:
            # Martyrs' Day.
            self._add_observed(self._add_holiday_jan_9(tr("Día de los Mártires")))

        if self._year <= 1971:
            # Constitution Day.
            self._add_holiday_mar_1(tr("Día de la Constitución"))

        # Carnival Tuesday.
        self._add_carnival_tuesday(tr("Martes de Carnaval"))

        # Good Friday.
        self._add_good_friday(tr("Viernes Santo"))

        # Labor Day.
        self._add_observed(self._add_labor_day(tr("Día del Trabajo")))

        # Separation Day.
        self._add_observed(self._add_holiday_nov_3(tr("Separación de Panamá de Colombia")))

        # Law #55 of Nov 7, 2001.
        if self._year >= 2002:
            # Colon Day.
            self._add_observed(self._add_holiday_nov_5(tr("Día de Colón")))

        # Cabinet Decree #347 of 1969.
        if self._year >= 1969:
            # Los Santos Uprising Day.
            self._add_observed(self._add_holiday_nov_10(tr("Primer Grito de Independencia")))

        # Independence Day.
        self._add_observed(self._add_holiday_nov_28(tr("Independencia de Panamá de España")))

        # Mother's Day.
        self._add_observed(self._add_holiday_dec_8(tr("Día de la Madre")))

        if self._year >= 2022:
            # National Mourning Day.
            self._add_observed(self._add_holiday_dec_20(tr("Día de Duelo Nacional")))

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Navidad")))

    def _populate_bank_holidays(self):
        # Carnival Monday.
        self._add_carnival_monday(tr("Lunes de Carnaval"))

        # National Symbols Day.
        self._add_holiday_nov_4(tr("Día de los Símbolos Patrios"))


class PA(Panama):
    pass


class PAN(Panama):
    pass


class PanamaStaticHolidays:
    # Presidential Inauguration Day.
    presidential_inauguration_day = tr("Toma posesión del Presidente de la república")
    special_public_holidays = {
        2014: (JUL, 1, presidential_inauguration_day),
        2019: (JUL, 1, presidential_inauguration_day),
        2024: (JUL, 1, presidential_inauguration_day),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.burkina_faso import BurkinaFaso
from tests.common import CommonCountryTests


class TestBurkinaFaso(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(BurkinaFaso)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_revolution_day(self):
        name = "Revolution Day"
        self.assertHolidayName(name, (f"{year}-01-03" for year in range(1967, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1967))
        obs_dts = (
            "2010-01-04",
            "2016-01-04",
            "2021-01-04",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_womens_day(self):
        name = "International Women's Day"
        self.assertHolidayName(name, (f"{year}-03-08" for year in self.full_range))
        obs_dts = (
            "2015-03-09",
            "2020-03-09",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2011-05-02",
            "2016-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-08-05" for year in self.full_range))
        obs_dts = (
            "2012-08-06",
            "2018-08-06",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_assumption_day(self):
        name = "Assumption Day"
        self.assertHolidayName(name, (f"{year}-08-15" for year in self.full_range))
        obs_dts = (
            "2010-08-16",
            "2021-08-16",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_martyrs_day(self):
        name = "Martyrs' Day"
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name, (f"{year}-10-31" for year in range(2016, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2016))
        obs_dts = ("2021-11-01",)
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHolidayName(name_observed, obs_dts)

    def test_all_saints_day(self):
        name = "All Saints' Day"
        self.assertHolidayName(name, (f"{year}-11-01" for year in self.full_range))
        obs_dts = (
            "2015-11-02",
            "2020-11-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_proclamation_of_independence_day(self):
        name = "Proclamation of Independence Day"
        self.assertHolidayName(name, (f"{year}-12-11" for year in self.full_range))
        obs_dts = (
            "2011-12-12",
            "2022-12-12",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        self.assertHolidayName(
            name,
            "2014-07-29",
            "2015-07-18",
            "2016-07-07",
            "2017-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertHolidayName(
            name,
            "2014-10-05",
            "2015-09-24",
            "2016-09-13",
            "2017-09-02",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophets_birthday(self):
        name = "Mawlid"
        self.assertHolidayName(
            name,
            "2014-01-14",
            "2015-01-03",
            "2015-12-24",
            "2016-12-12",
            "2017-12-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "Revolution Day"),
            ("2022-03-08", "International Women's Day"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labour Day"),
            ("2022-05-02", "Eid al-Fitr; Labour Day (observed)"),
            ("2022-05-26", "Ascension Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-08-05", "Independence Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-10-09", "Mawlid"),
            ("2022-10-31", "Martyrs' Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-11", "Proclamation of Independence Day"),
            ("2022-12-12", "Proclamation of Independence Day (observed)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "Revolution Day"),
            ("2022-03-08", "International Women's Day"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Eid al-Fitr; Labor Day (observed)"),
            ("2022-05-26", "Ascension Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-08-05", "Independence Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-10-09", "Prophet's Birthday"),
            ("2022-10-31", "Martyrs' Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-11", "Proclamation of Independence Day"),
            ("2022-12-12", "Proclamation of Independence Day (observed)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2022-01-01", "Jour de l'An"),
            ("2022-01-03", "Soulèvement populaire"),
            ("2022-03-08", "Journée internationale de la femme"),
            ("2022-04-18", "Lundi de Pâques"),
            ("2022-05-01", "Fête du travail"),
            ("2022-05-02", "Fête du travail (observé); Jour de Ramadan"),
            ("2022-05-26", "Ascension"),
            ("2022-07-09", "Jour de Tabaski"),
            ("2022-08-05", "Fête nationale"),
            ("2022-08-15", "Assomption"),
            ("2022-10-09", "Mouloud"),
            ("2022-10-31", "Journée nationale des martyrs"),
            ("2022-11-01", "Toussaint"),
            ("2022-12-11", "Proclamation de l'Indépendance"),
            ("2022-12-12", "Proclamation de l'Indépendance (observé)"),
            ("2022-12-25", "Jour de Noël"),
            ("2022-12-26", "Jour de Noël (observé)"),
        )



#!/usr/bin/env python3


#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import argparse
import re
import sys
from datetime import datetime, timezone
from pathlib import Path

from git import Repo
from github import Auth, Github
from github.GithubException import UnknownObjectException

sys.path.insert(0, str(Path.cwd()))
import holidays

BRANCH_NAME = "dev"
HEADER_TEMPLATE = """
## Version {version}

Released {month} {day}, {year}
"""
IGNORED_CONTRIBUTORS = {"dependabot[bot]", "github-actions[bot]"}
REPOSITORY_NAME = "vacanza/holidays"


class ReleaseNotesGenerator:
    """Generates release notes based on local git commits and GitHub PRs metadata.

    Usage example: scripts/generate_release_notes.py
    """

    def __init__(self) -> None:
        arg_parser = argparse.ArgumentParser()
        arg_parser.add_argument(
            "-a",
            "--author-only",
            action="extend",
            default=[],
            help="Add only author as a contributor for this PR",
            nargs="+",
            type=int,
        )
        arg_parser.add_argument(
            "-c",
            "--cut-off-at",
            help="Cut off at PR",
            required=False,
            type=int,
        )
        arg_parser.add_argument(
            "-e",
            "--exclude",
            action="extend",
            default=[],
            help="Exclude this PR from the release notes",
            nargs="+",
            type=int,
        )
        arg_parser.add_argument(
            "-v",
            "--verbose",
            action="store_true",
            default=False,
            help="Verbose output",
        )
        self.args = arg_parser.parse_args()

        self.local_repo = Repo(Path.cwd())
        self.remote_repo = Github(auth=Auth.Token(self.github_token)).get_repo(REPOSITORY_NAME)

        self.previous_commits: set[str] = set()
        self.pull_requests: dict[int, tuple[str, str]] = {}
        self.skipped_pull_requests: set[int] = set()

        self.tag = holidays.__version__

        try:
            latest_tag = self.remote_repo.get_tags()[0]
            self.latest_tag_name = latest_tag.name
            self.previous_commits.add(latest_tag.commit.sha)
        except IndexError:
            self.latest_tag_name = None

    @property
    def github_token(self, path=Path(".github_token")):
        """Return GitHub access token."""
        return path.read_text(encoding="UTF-8").strip()

    @property
    def is_ready(self):
        """Perform environment checks and input validation."""
        current_branch = str(self.local_repo.active_branch)
        if current_branch != BRANCH_NAME:
            exit(
                f"Switch to '{BRANCH_NAME}' first (currently in "
                f"'{current_branch}'). Use 'git switch {BRANCH_NAME}'."
            )

        return True

    @property
    def sorted_pull_requests(self):
        def custom_order(pr):
            if re.findall(r"^(Introduce|Refactor)", pr[0]) or re.findall(r"Add .* support", pr[0]):
                weight = 10
            elif re.findall(r"^Add .* holidays$", pr[0]):
                weight = 20
            elif re.findall(r"(^Localize)|(localization$)", pr[0]):
                weight = 30
            elif re.findall(r"^Fix", pr[0]):
                weight = 40
            elif re.findall(r"^(Change|Improve|Optimize|Update|Upgrade)", pr[0]):
                weight = 50
            else:
                weight = 100

            return weight, pr

        return sorted(self.pull_requests.values(), key=custom_order)

    def add_pull_request(self, pull_request):
        """Add pull request information to the release notes dict."""
        author = pull_request.user.login if pull_request.user else None
        pr_number = pull_request.number
        if author in IGNORED_CONTRIBUTORS:
            print(f"Skipping #{pr_number} {pull_request.title} by {author}")
            self.skipped_pull_requests.add(pr_number)
            return None

        # Skip failed release attempt PRs, version upgrades.
        pr_title = pull_request.title
        if pr_title.startswith(("v", "Bump", "Revert", "chore:")):
            self.skipped_pull_requests.add(pr_number)
            return None

        # Get contributors (expand from commits by default).
        contributors = set()
        if pr_number not in self.args.author_only:
            for commit in pull_request.get_commits():
                if (
                    commit.author
                    and (author_login := commit.author.login) not in IGNORED_CONTRIBUTORS
                ):
                    contributors.add(author_login)

        contributors = (f"@{c}" for c in [author, *sorted(contributors - {author}, key=str.lower)])
        self.pull_requests[pr_number] = (pr_title, f"#{pr_number} by {', '.join(contributors)}")

    def generate_release_notes(self):
        """Generate release notes contents."""
        print("Processing pull requests...")
        self.get_new_pull_requests()
        self.get_old_pull_requests()
        print("Done!")

    def get_new_pull_requests(self):
        """Get PRs created after the latest release.

        This operation also populates a set of previous release commits.
        """
        cut_off_at = self.args.cut_off_at
        excluded_pr_numbers = set(self.args.exclude)
        for pull_request in self.remote_repo.get_pulls(state="closed"):
            # Stop getting pull requests after previous release tag or specific PR number.
            cut_off = cut_off_at and pull_request.number == cut_off_at
            if cut_off or pull_request.title == self.latest_tag_name:
                # Get previous release commits SHAs.
                for commit in pull_request.get_commits():
                    self.previous_commits.add(commit.sha)
                break

            # Skip unrelated PRs (unmerged, wrong branch).
            if not pull_request.merged or pull_request.base.ref != BRANCH_NAME:
                continue

            if pull_request.number in excluded_pr_numbers:
                if self.args.verbose:
                    print(f"Excluding PR #{pull_request.number} as requested")
                continue

            if self.args.verbose:
                messages = [f"Fetching PR #{pull_request.number}"]
                if pull_request.number in self.args.author_only:
                    messages.append("(keeping PR author as a sole contributor)")
                print(" ".join(messages))

            self.add_pull_request(pull_request)

    def get_old_pull_requests(self):
        """Get PRs created before the latest release."""
        pull_request_numbers = set()
        for commit in self.local_repo.iter_commits():
            if commit.hexsha in self.previous_commits:
                break

            try:
                pull_request_number = re.findall(r"#(\d{3,})", commit.message)[0]
                pull_request_numbers.add(int(pull_request_number))
            except IndexError:
                continue

        # Fetch old PRs metadata only. Skip all known PRs.
        pull_request_numbers -= set(self.pull_requests.keys())
        pull_request_numbers -= set(self.args.exclude)
        pull_request_numbers -= self.skipped_pull_requests
        for pull_request_number in pull_request_numbers:
            if self.args.verbose:
                messages = [f"Fetching PR #{pull_request_number}"]
                if pull_request_number in self.args.author_only:
                    messages.append("(keeping PR author as a sole contributor)")
                print(" ".join(messages))

            try:
                self.add_pull_request(self.remote_repo.get_pull(pull_request_number))
            # 3rd party contributions to forks.
            except UnknownObjectException:
                pass

    def print_release_notes(self):
        """Print generated release notes."""
        print("")
        if self.pull_requests:
            today = datetime.now(tz=timezone.utc)
            print(
                HEADER_TEMPLATE.format(
                    day=today.day, month=today.strftime("%B"), version=self.tag, year=today.year
                )
            )
            print("\n".join(f"- {pr[0]} ({pr[1]})" for pr in self.sorted_pull_requests))

        else:
            print(f"No changes since {self.latest_tag_name} release.")


if __name__ == "__main__":
    rng = ReleaseNotesGenerator()
    if rng.is_ready:
        rng.generate_release_notes()
        rng.print_release_notes()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.falkland_islands import FalklandIslands
from tests.common import CommonCountryTests


class TestFalklandIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(FalklandIslands)

    def test_special_holidays(self):
        self.assertHoliday(
            "2022-09-19",
            "2023-05-08",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dt = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_margaret_thatcher_day(self):
        name = "Margaret Thatcher Day"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-01-10" for year in self.full_range))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_queens_birthday(self):
        name = "HM The Queen's Birthday"
        self.assertHolidayName(name, (f"{year}-04-21" for year in range(self.start_year, 2023)))
        self.assertNoHolidayName(name, range(2023, self.end_year))
        obs_dt = (
            "2007-04-23",
            "2012-04-23",
            "2013-04-22",
            "2018-04-23",
            "2019-04-22",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_liberation_day(self):
        name = "Liberation Day"
        self.assertHolidayName(name, (f"{year}-06-14" for year in self.full_range))
        obs_dt = (
            "2008-06-16",
            "2009-06-15",
            "2014-06-16",
            "2015-06-15",
            "2020-06-15",
            "2025-06-16",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_falkland_day(self):
        name = "Falkland Day"
        self.assertHolidayName(name, (f"{year}-08-14" for year in range(self.start_year, 2002)))
        self.assertNoHolidayName(name, range(2002, self.end_year))
        obs_dt = (
            "1983-08-15",
            "1988-08-15",
            "1993-08-16",
            "1994-08-15",
            "1999-08-16",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)
        self.assertWorkdayHolidayName(
            name, (f"{year}-08-14" for year in range(2002, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2002))

    def test_peat_cutting_day(self):
        name = "Peat Cutting Day"
        self.assertHolidayName(
            name,
            "2020-10-05",
            "2021-10-04",
            "2022-10-03",
            "2023-10-02",
            "2024-10-07",
            "2025-10-06",
        )
        self.assertHolidayName(name, range(2002, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2002))

    def test_kings_birthday(self):
        name = "HM The King's Birthday"
        self.assertHolidayName(name, (f"{year}-11-14" for year in range(2022, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2022))
        obs_dt = (
            "2026-11-16",
            "2027-11-15",
            "2032-11-15",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dt = (
            "2010-12-28",
            "2011-12-28",
            "2016-12-28",
            "2021-12-28",
            "2022-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dt = (
            "2010-12-29",
            "2015-12-28",
            "2020-12-28",
            "2021-12-29",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_christmas_day_three(self):
        name = "Christmas Holiday"
        self.assertHolidayName(name, (f"{year}-12-27" for year in self.full_range))
        obs_dt = (
            "2014-12-29",
            "2015-12-29",
            "2020-12-29",
            "2025-12-29",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_government_holiday(self):
        name = "Government Holiday"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(
            name,
            "2022-12-29",
            "2022-12-30",
            "2023-12-28",
            "2023-12-29",
            "2024-12-30",
            "2024-12-31",
            "2025-12-30",
            "2025-12-31",
        )
        self.assertGovernmentHolidayName(name, self.full_range)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-01-10", "Margaret Thatcher Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-06-14", "Liberation Day"),
            ("2025-06-16", "Liberation Day (observed)"),
            ("2025-08-14", "Falkland Day"),
            ("2025-10-06", "Peat Cutting Day"),
            ("2025-11-14", "HM The King's Birthday"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
            ("2025-12-27", "Christmas Holiday"),
            ("2025-12-29", "Christmas Holiday (observed)"),
            ("2025-12-30", "Government Holiday"),
            ("2025-12-31", "Government Holiday"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-10", "Margaret Thatcher Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-06-14", "Liberation Day"),
            ("2025-06-16", "Liberation Day (observed)"),
            ("2025-08-14", "Falkland Day"),
            ("2025-10-06", "Peat Cutting Day"),
            ("2025-11-14", "King's Birthday"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
            ("2025-12-27", "Christmas Holiday"),
            ("2025-12-29", "Christmas Holiday (observed)"),
            ("2025-12-30", "Government Holiday"),
            ("2025-12-31", "Government Holiday"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.cocos_islands import CocosIslands
from tests.common import CommonCountryTests


class TestCocosIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(CocosIslands)

    def test_special_holidays(self):
        self.assertHoliday(
            "2022-09-22",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_australia_day(self):
        name = "Australia Day"
        self.assertHolidayName(name, (f"{year}-01-26" for year in self.full_range))
        obs_dts = (
            "2013-01-28",
            "2014-01-27",
            "2019-01-28",
            "2020-01-27",
            "2025-01-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_act_of_self_determination_day(self):
        name = "Act of Self Determination Day"
        self.assertHolidayName(
            name,
            "2007-04-05",
            (
                f"{year}-04-06"
                for year in (*range(self.start_year, 2007), *range(2008, self.end_year))
            ),
        )
        obs_dts = (
            "2013-04-08",
            "2014-04-07",
            "2019-04-10",
            "2024-04-08",
            "2025-04-07",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_anzac_day(self):
        name = "ANZAC Day"
        self.assertHolidayName(name, (f"{year}-04-25" for year in self.full_range))
        obs_dts = (
            "2009-04-27",
            "2010-04-26",
            "2015-04-27",
            "2020-04-27",
            "2021-04-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_sovereigns_birthday(self):
        name_queen = "Queen's Birthday"
        name_king = "King's Birthday"
        self.assertHolidayName(
            name_queen,
            "2018-06-11",
            "2019-06-10",
            "2020-06-08",
            "2021-06-07",
            "2022-06-06",
        )
        self.assertHolidayName(
            name_king,
            "2023-06-06",
            "2024-06-06",
            "2025-06-09",
        )
        self.assertHolidayName(name_queen, range(self.start_year, 2023))
        self.assertHolidayName(name_king, range(2023, self.end_year))
        self.assertNoHolidayName(name_queen, range(2023, self.end_year))
        self.assertNoHolidayName(name_king, range(self.start_year, 2023))

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2015-12-28",
            "2016-12-27",
            "2020-12-28",
            "2021-12-28",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-26",
            "2016-12-26",
            "2021-12-27",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)

    def test_islamic_new_year(self):
        name = "Islamic New Year"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2015-10-14",
            "2016-10-03",
            "2017-09-22",
            "2018-09-11",
            "2019-09-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(self.start_year, 2020))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(2020, self.end_year))
        obs_dts = (
            "1999-04-19",
            "2004-02-23",
            "2011-11-28",
            "2014-10-27",
            "2019-09-02",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_prophets_birthday(self):
        name = "Prophet's Birthday"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-19",
            "2022-10-08",
            "2023-09-27",
            "2024-09-16",
            "2025-09-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2004-05-03",
            "2012-02-06",
            "2015-01-05",
            "2019-11-11",
            "2022-10-10",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-03",
            "2023-04-21",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2004-11-15",
            "2012-08-20",
            "2017-06-26",
            "2020-05-25",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-17",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2011-11-07",
            "2014-10-06",
            "2019-08-12",
            "2022-07-11",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-26", "Australia Day"),
            ("2022-04-06", "Act of Self Determination Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "ANZAC Day"),
            ("2022-05-03", "Eid al-Fitr"),
            ("2022-06-06", "Queen's Birthday"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-07-11", "Eid al-Adha (observed)"),
            ("2022-09-22", "National Day of Mourning for Queen Elizabeth II"),
            ("2022-10-08", "Prophet's Birthday"),
            ("2022-10-10", "Prophet's Birthday (observed)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day; Christmas Day (observed)"),
            ("2022-12-27", "Boxing Day (observed)"),
        )

    def test_l10n_coa_cc(self):
        self.assertLocalizedHolidays(
            "coa_CC",
            ("2022-01-01", "Hari Tahun Baru"),
            ("2022-01-03", "Hari Tahun Baru (disambut)"),
            ("2022-01-26", "Hari Australia"),
            ("2022-04-06", "Hari Penentuan Diri"),
            ("2022-04-15", "Jumat Agung"),
            ("2022-04-18", "Isnin Paskah"),
            ("2022-04-25", "Hari ANZAC"),
            ("2022-05-03", "Hari Raya Puasa"),
            ("2022-06-06", "Hari Ulang Tahun Ratu"),
            ("2022-07-09", "Hari Raya Haji"),
            ("2022-07-11", "Hari Raya Haji (disambut)"),
            ("2022-09-22", "Hari Berkabung Negara untuk Ratu Elizabeth II"),
            ("2022-10-08", "Hari Maulaud Nabi"),
            ("2022-10-10", "Hari Maulaud Nabi (disambut)"),
            ("2022-12-25", "Hari Natal"),
            ("2022-12-26", "Hari Boxing; Hari Natal (disambut)"),
            ("2022-12-27", "Hari Boxing (disambut)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-26", "Australia Day"),
            ("2022-04-06", "Act of Self Determination Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "ANZAC Day"),
            ("2022-05-03", "Eid al-Fitr"),
            ("2022-06-06", "Queen's Birthday"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-07-11", "Eid al-Adha (observed)"),
            ("2022-09-22", "National Day of Mourning for Queen Elizabeth II"),
            ("2022-10-08", "Prophet's Birthday"),
            ("2022-10-10", "Prophet's Birthday (observed)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day; Christmas Day (observed)"),
            ("2022-12-27", "Boxing Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import ARMED_FORCES, PUBLIC
from holidays.countries.palau import Palau
from tests.common import CommonCountryTests


class TestPalau(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Palau)

    def test_special_holidays(self):
        self.assertHoliday("2020-11-03")
        self.assertArmedForcesHoliday("2020-11-11")
        self.assertHalfDayHoliday("2019-09-30")

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2010-12-31",
            "2012-01-02",
            "2017-01-02",
            "2021-12-31",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_youth_day(self):
        name = "Youth Day"
        self.assertHolidayName(name, (f"{year}-03-15" for year in self.full_range))
        obs_dts = (
            "2009-03-16",
            "2014-03-14",
            "2015-03-16",
            "2020-03-16",
            "2025-03-14",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_senior_citizens_day(self):
        name = "Senior Citizens Day"
        self.assertHolidayName(name, (f"{year}-05-05" for year in self.full_range))
        obs_dts = (
            "2012-05-04",
            "2013-05-06",
            "2018-05-04",
            "2019-05-06",
            "2024-05-06",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_memorial_day(self):
        name = "Memorial Day"
        self.assertHolidayName(name, "2011-05-30", "2012-05-28")
        self.assertNoHolidayName(name, range(self.start_year, 2011), range(2013, self.end_year))

    def test_presidents_day(self):
        name = "President's Day"
        self.assertHolidayName(name, (f"{year}-06-01" for year in range(2018, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2018))
        obs_dts = (
            "2019-05-31",
            "2024-05-31",
            "2025-06-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_constitution_day(self):
        name = "Constitution Day"
        self.assertHolidayName(name, (f"{year}-07-09" for year in self.full_range))
        obs_dts = (
            "2011-07-08",
            "2016-07-08",
            "2017-07-10",
            "2022-07-08",
            "2023-07-10",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_labor_day(self):
        name = "Labor Day"
        self.assertHolidayName(
            name,
            "2020-09-07",
            "2021-09-06",
            "2022-09-05",
            "2023-09-04",
            "2024-09-02",
            "2025-09-01",
        )
        self.assertHolidayName(name, self.full_range)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-10-01" for year in range(2018, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2018))
        obs_dts = (
            "2022-09-30",
            "2023-10-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_united_nations_day(self):
        name = "United Nations Day"
        self.assertHolidayName(name, (f"{year}-10-24" for year in self.full_range))
        obs_dts = (
            "2009-10-23",
            "2010-10-25",
            "2015-10-23",
            "2020-10-23",
            "2021-10-25",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_thanksgiving_day(self):
        name = "Thanksgiving Day"
        self.assertHolidayName(
            name,
            "2020-11-26",
            "2021-11-25",
            "2022-11-24",
            "2023-11-23",
            "2024-11-28",
            "2025-11-27",
        )
        self.assertHolidayName(name, self.full_range)

    def test_family_day(self):
        name = "Family Day"
        self.assertHolidayName(
            name,
            "2020-11-27",
            "2021-11-26",
            "2022-11-25",
            "2023-11-24",
            "2024-11-22",
            "2025-11-28",
        )
        self.assertHolidayName(name, range(2017, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2017))

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-24",
            "2011-12-26",
            "2016-12-26",
            "2021-12-24",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2020(self):
        # https://www.facebook.com/photo/?fbid=1499007180251376&set=a.175933635892077
        self.assertHolidays(
            Palau(categories=(ARMED_FORCES, PUBLIC), years=2020),
            ("2020-01-01", "New Year's Day"),
            ("2020-03-15", "Youth Day"),
            ("2020-03-16", "Youth Day (observed)"),
            ("2020-05-05", "Senior Citizens Day"),
            ("2020-06-01", "President's Day"),
            ("2020-07-09", "Constitution Day"),
            ("2020-09-07", "Labor Day"),
            ("2020-10-01", "Independence Day"),
            ("2020-10-23", "United Nations Day (observed)"),
            ("2020-10-24", "United Nations Day"),
            ("2020-11-03", "National Day of Democracy"),
            ("2020-11-11", "Veterans Day"),
            ("2020-11-26", "Thanksgiving Day"),
            ("2020-11-27", "Family Day"),
            ("2020-12-25", "Christmas Day"),
        )

    def test_2022(self):
        # https://www.facebook.com/photo/?fbid=2076492362502852&set=a.175933635892077
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-03-15", "Youth Day"),
            ("2022-05-05", "Senior Citizens Day"),
            ("2022-06-01", "President's Day"),
            ("2022-07-08", "Constitution Day (observed)"),
            ("2022-07-09", "Constitution Day"),
            ("2022-09-05", "Labor Day"),
            ("2022-09-30", "Independence Day (observed)"),
            ("2022-10-01", "Independence Day"),
            ("2022-10-24", "United Nations Day"),
            ("2022-11-24", "Thanksgiving Day"),
            ("2022-11-25", "Family Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )

    def test_2023(self):
        # https://www.facebook.com/photo/?fbid=510442897789296&set=a.368725625294358
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-03-15", "Youth Day"),
            ("2023-05-05", "Senior Citizens Day"),
            ("2023-06-01", "President's Day"),
            ("2023-07-09", "Constitution Day"),
            ("2023-07-10", "Constitution Day (observed)"),
            ("2023-09-04", "Labor Day"),
            ("2023-10-01", "Independence Day"),
            ("2023-10-02", "Independence Day (observed)"),
            ("2023-10-24", "United Nations Day"),
            ("2023-11-23", "Thanksgiving Day"),
            ("2023-11-24", "Family Day"),
            ("2023-12-25", "Christmas Day"),
        )
